import { declare } from '@babel/helper-plugin-utils';
import { types } from '@babel/core';

const TRACE_ID = "__self";
function getThisFunctionParent(path) {
  let scope = path.scope;
  do {
    const {
      path
    } = scope;
    if (path.isFunctionParent() && !path.isArrowFunctionExpression()) {
      return path;
    }
  } while (scope = scope.parent);
  return null;
}
function isDerivedClass(classPath) {
  return classPath.node.superClass !== null;
}
function isThisAllowed(path) {
  const parentMethodOrFunction = getThisFunctionParent(path);
  if (parentMethodOrFunction === null) {
    return true;
  }
  if (!parentMethodOrFunction.isMethod()) {
    return true;
  }
  if (parentMethodOrFunction.node.kind !== "constructor") {
    return true;
  }
  return !isDerivedClass(parentMethodOrFunction.parentPath.parentPath);
}
var index = declare(api => {
  api.assertVersion("8.0.0-alpha.14");
  const visitor = {
    JSXOpeningElement(path) {
      if (!isThisAllowed(path)) {
        return;
      }
      const node = path.node;
      const id = types.jsxIdentifier(TRACE_ID);
      const trace = types.thisExpression();
      node.attributes.push(types.jsxAttribute(id, types.jsxExpressionContainer(trace)));
    }
  };
  return {
    name: "transform-react-jsx-self",
    visitor: {
      Program(path) {
        path.traverse(visitor);
      }
    }
  };
});

export { index as default };
//# sourceMappingURL=index.js.map
